part of tags.html;

class EditableLabelResources extends Resources {
  String get style =>
"""
input[type="text"] {
  background-color:transparent;
  border-style:none;
  color:${theme.colors.text};
  font-family:${theme.fonts.family};
  font-size:${theme.fonts.size};
}
""";
  const EditableLabelResources();
}

class EditableLabel extends DivElement with Themeable {
 
  static const TAG = 'editable-label';
  
  EditableLabelResources resources = const EditableLabelResources();
  final StyleElement styleElement = new StyleElement();
  StreamSubscription _themeListener;
  
  ShadowRoot _shadow;
  LabelElement _label;
  TextInputElement _textInput;
  
  String _text;
  String _lastText;
  
  Function displayFilter;
  
  bool _isEditing = false;
  /// Whether or not this editable label is editing.
  bool get isEditing => _isEditing;
  
  final _onTextChange = new StreamController<String>.broadcast();
  Stream<String> get onTextChange => _onTextChange.stream;
  
  final _onEditChange = new StreamController<bool>.broadcast();
  /// An observable stream of this editable label's state transitions.
  /// 
  /// A `true` value represents a transition to an [isEditing] value of `true`.
  Stream<bool> get onEditChange => _onEditChange.stream;
  
  String 
    get text => _text;
    set text(String t) {
      if (_textInput.value != t) {
        _textInput.value = t;
      }
      String displayText = t;
      if (displayFilter != null) {
        displayText = displayFilter(t);
      }
      if (_label.text != displayText) {
        _label.text = displayText;
      }
      if (_text != t) {
        _text = t;
        _onTextChange.add(t);
      }
    }  
  
  factory EditableLabel() => new Element.tag('div', TAG);
  
  EditableLabel.created() : super.created() {
    children(this, () {      
      _shadow = shadow(() {
        append(styleElement..text = resources.style);
        _label = label()
        ..style.height = '100%'
        ..style.width = '100%';        
      });
    });
    _textInput = new InputElement()
    ..style.height = '100%'
    ..style.width = '100%'
    ..onBlur.listen((_) {      
      cancelEdit();
    })
    ..onDoubleClick.listen((Event ev) {
      ev.stopPropagation();      
    })
    ..onKeyUp.listen((KeyboardEvent ev) {
      if (ev.keyCode == KeyCode.ESC) {        
        cancelEdit();
      } else if (ev.keyCode == KeyCode.ENTER) {
        completeEdit();
      }
    })
    ..type = 'text';
  }
    
  void enteredView() {
    super.enteredView();
    _themeListener = onThemeChange.listen((_) => refreshStyle());
    refreshStyle();
  }
  
  void leftView() {
    super.leftView();
    _themeListener.cancel();
  }
  
  void beginEdit({bool select: true}) {
    if (!isEditing) {
      _lastText = this.text;    
      _shadow.children[1] = _textInput;
      _textInput.focus();
      if (select) {
        _textInput.select();
      }
      _onEditChange.add(_isEditing = true);
    }
  }
  
  void cancelEdit() {
    if (isEditing) {
      this.text = _lastText;
      _shadow.children[1] = _label;
      _onEditChange.add(_isEditing = false);
    }
  }
  
  void completeEdit() {
    if (isEditing) {
      this.text = _textInput.value;
      _lastText = this.text;
      _shadow.children[1] = _label;
      _onEditChange.add(_isEditing = false);
    }
  }
}
